# 隣接行列を読み込んでおく。
G <- matrix(
  c(0, 1, 0, 0, 1, 0,
    1, 0, 1, 0, 1, 0,
    0, 1, 0, 0, 1, 1,
    0, 0, 0, 0, 1, 0,
    1, 1, 1, 1, 0, 0,
    0, 0, 1, 0, 0, 0), nrow=6, ncol=6, byrow =T)

# snaを読み込む。
library(sna)


# 矢印は示さないグラフ
gplot(G, gmode = "graph", label =c(1:ncol(G)))


##############################################################
# 固有ベクトル中心性とボナチッチ・パワー中心性の関係を確認する。

# snaでのコマンドでの固有ベクトル中心性を得る
myevc <- evcent(G)
myevc


# 固有値関係の情報を得る。
myeigen <- eigen(G)
# 固有値をえる。
myeigen$values
# 固有値の絶対値（固有値は値の順に並んでいる）
abs(myeigen$values)

# # 固有ベクトル（固有値の順に並ぶ）を求めたかったら以下を行う。
# myeigen$vectors


# 最大固有値は2.7537148だから、
# b < 1/2.7537148
# であり、

1/2.7537148

# 1/2.7537148は0.3631458
# なので、なるべく大きなbとして、以下とする。

b <- 0.363145

# snaのコマンドからボナチッチ・パワー中心性を得る。
mybona <- bonpow(G, exponent = b, rescale=T)
mybona

# 以上は総和1というように値を調整しているので、
# 固有ベクトル中心性も値を調整した以下と比較し、
# 近い値となっていることを確認できる。
mybona
myevc/sum(myevc)



##############################################################
# 次数中心性、拡散中心性、ボナチッチ・パワー中心性の関係を確認する。

###########################
# 次数中心性を求める。
# snaのコマンドからは以下で。
degree(G, gmode="graph")

# mydegcとして保存
mydegc <- degree(G, gmode="graph")
mydegc

###########################
# 拡散中心性を求める。

# はじめに全ての要素を1とする縦ベクトルを作っておく。
one <-matrix(
  c(1,
    1,
    1,
    1,
    1,
    1)
)

##############################
# 拡散中心性を求める。
# bの値は、先ほど用いた値を流用する。
# T=1
myDC1 <- b*G %*% one
myDC1

# 総和を1として調整して比較をし、同じになることを確認する。
t(myDC1/sum(myDC1)) # 見やすいように転置している。
mydegc/sum(mydegc)


# T=4の時のDiffusion中心性は以下となる。
myDC4 <- (b*G %*% one) +b *((b*G) %*% (b*G) %*% one) +((b*G) %*% (b*G) %*% (b*G) %*% one)+((b*G) %*% (b*G) %*% (b*G) %*% (b*G) %*% one)
myDC4
myDC4/sum(myDC4) # 総和を1として標準化

# 参考までに、次数中心性、T=4のDiffusion中心性、ボナチッチ・パワー中心性を並べた。
# おおよそ、次数中心性からボナチッチ・パワー中心性へ近づいていることが確認できる。
mydegc/sum(mydegc)
t(myDC4/sum(myDC4)) #見やすいように転置させた。
mybona/sum(mybona)

