# パッケージのインストールの仕方は、
# Googleで"RStudio パッケージ　インストール"でやり方を確認するとよい。
# 基本は以下のやり方である：
# やり方(1): 右下のwindowで、package -> install -> "sna"など、パッケージの名前に入力
# やり方(2)以下のコマンドでインストールする。
# install.packages("sna")
# install.packages("igraph")

# RStudio上でのコマンドの実行はWindowsではコントロールキーとエンターキーの同時押し、
# Macではコマンドキーとエンターキーの同時押しを行えばよい。


#detach(package:igraph) # igraphを既に読み込んでいた場合は外す。競合があるので。

library(sna)

# はじめに点と線での表現で考える。
# 線には点の情報も含まれるので、線だけ定義したらよい。
# 点の名称は、もっとも単純なものとして、番号とすればよい。

# 以下では、
# (1) ネットワークを定義して、
# (2) ネットワークのグラフを描画して
# (3) 隣接行列および線の集合としてどのように、保存されているかを確認する
# ということを繰り返しているだけである。

# テキストで述べたことが基本であるが、それぞれのパッケージやコマンドで癖があるので、
# 冗長であるがいろいろな場合で確認をした。




# ●方向のない線を考える。
# はじめに、線の集合を定義する。
myedgelist <- rbind(c(1, 2))

# 以下でコマンドのオプションの説明は省いているが、
# 基本的に「?コマンド名」でヘルプが表示され、コマンドやその中のオプションを確認できる。

# 以上では
?c
# とすると、cは数値を合わせて、行ベクトルを作るコマンドということがわかる。

#
?rbind
# とすると、rbind は行ベクトルを結合してデータを作るコマンドということがわかる。
# ここでは一つの行ベクトルしか出ていないが、後の例で複数の行ベクトルを結合する。


# 次に、ネットワークオブジェクトを作る。
# ここで、方向性がないことを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=F)

# 矢印は示さないネットワークグラフの描画
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示したネットワークグラフの描画（双方向となる）
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) #隣接行列がテキストで学んだ形となっている。
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、点の順序関係ない。
# 最初の情報が線についての情報である。つづく、attrは点の総数と、つづいて点の名前が示されている。

# ●方向のある線を考える。
# 線の集合を定義している
myedgelist <- rbind(c(1, 2))
# 以下でネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=T)
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet)
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、順序関係あり。

# 以下のように矢印を示さないことも可能だが、
# 実質的には誤解を与える表現といえる。
gplot(mynet, gmode = "graph", label =c(1, 2))


# ●方向のないグラフでは、線を(1, 2)から(2, 1)へ方向を逆にして、定義しても結果は同じである。
# 線の集合を定義している
myedgelist <- rbind(c(2, 1)) #ここで先とは異なりc(2, 1)となっていることに注意する。
# ここでネットワークオブジェクトを作っている。
# 方向性がないことを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=F)

# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) #隣接行列がテキストで学んだ形となっている。
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、順序関係ないので(1, 2)という線になっている。


# ●方向のある線を考える。
# 線の集合を定義している
myedgelist <- rbind(c(1, 2))
# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=T)
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet)
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、順序関係あり。

# [注意]以下のように矢印を示さないことも可能だが、
# 実質的には誤解を与える表現といえる。
gplot(mynet, gmode = "graph", label =c(1, 2))


# ●方向のある線を考える。
# 線の集合を定義している。ここでは、線を(1, 2)から(2, 1)へ方向を逆にして、結果が変わることを確認する。
myedgelist <- rbind(c(2, 1))
# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=T)
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet)
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、線として(2, 1)が示されている。
# 方向のないネットワークのように、(1, 2)に変換されることはなく、順序が関係していることがわかる。




# 二つの線で示しても良い。
# 方向は考えない
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 1))

# ここでネットワークオブジェクトを作っている。
# 方向性がないことを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=F)

# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) # テキストで学んだ対称行列の形となっている。
as.matrix(mynet, matrix.type = "edgelist") #方向のないネットワークでは、線はひとつに集約されている。



# 二つの線で示しても良い。
# 方向を考える。
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 1))

# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=T)

# 矢印は示さない（矢印のあるネットワークなのでこの表現は望ましくない。）
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) # 対称行列の形となっている。
as.matrix(mynet, matrix.type = "edgelist") #線は二つで示されている。

# 以上で、隣接行列だけでは、
# それが方向無しならば、線は{1, 2}で示され、
# それが方向有ならば、線は順序対の(1, 2)と(2, 1)で示される。
# つまり、隣接行列では含まれない情報がある。




# 図11のネットワークの描画
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 1),
                    c(3, 1),
                    c(3, 2),
                    c(4, 1),
                    c(4, 2),
                    c(5, 1),
                    c(5, 2)
)

# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=T)

# 矢印を示す。
gplot(mynet, gmode = "digraph", label =c(1, 2, 3, 4, 5))

# 隣接行列と線の集合の確認
as.matrix(mynet)
as.matrix(mynet, matrix.type = "edgelist")


# 図13のネットワークの描画
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 3),
                    c(3, 4),
                    c(4, 1),
                    c(2, 5))

# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- network(myedgelist, matrix.type="edgelist", directed=F)

# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5))

# 隣接行列と線の集合の確認
as.matrix(mynet) # 対称行列の形となっている。
as.matrix(mynet, matrix.type = "edgelist")





##################################
# Adjacency Matrix

# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(0, 0))
# 方向なしで、ここでネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=F) #ここで方向性を無いとしている。
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) #隣接行列が対称行列に変換されている。
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、順序関係ない。


# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(0, 0))
# 方向ありで、ここでネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=T)
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) # 指定した隣接行列のままである。
as.matrix(mynet, matrix.type = "edgelist") #線は隣接行列のとおり、1つとなっている。



# 離接行列を定義している
mynetmatrix <- rbind(c(0, 0),
                     c(1, 0))
# 方向なしで、ネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=F) #ここで方向性を無いとすると
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) #隣接行列が対称行列に変換されている。
as.matrix(mynet, matrix.type = "edgelist") #線は一つで、順序関係ない。



# 離接行列を定義している
mynetmatrix <- rbind(c(0, 0),
                     c(1, 0))
# 方向ありで、ネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=T)
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet) # 指定した行列のままである。
as.matrix(mynet, matrix.type = "edgelist") #順序対(2, 1)が示されている。



# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(1, 0))
# 方向なしで、ネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=F)
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet)
as.matrix(mynet, matrix.type = "edgelist") #線は一つに集約されている。



# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(1, 0))
# 方向ありで、ネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=T)
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2))
# 矢印を示す
gplot(mynet, gmode = "digraph", label =c(1, 2))

# 隣接行列と線の集合の確認
as.matrix(mynet)
as.matrix(mynet, matrix.type = "edgelist") #線は二つで示されている。



# 図1.11のグラフ
# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1, 0, 0, 0),
                     c(1, 0, 0, 0, 0),
                     c(1, 1, 0, 0, 0),
                     c(1, 1, 0, 0, 0),
                     c(1, 1, 0, 0, 0))
# 方向ありの、ネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=T)
# 矢印は示したネットワークグラフ
gplot(mynet, gmode = "digraph", label =c(1, 2, 3, 4, 5))

# 隣接行列と線の集合の確認
as.matrix(mynet) # 隣接行列
as.matrix(mynet, matrix.type = "edgelist") # 線の集合


# 図1.13のグラフ
# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1, 0, 1, 0),
                     c(0, 0, 1, 0, 1),
                     c(0, 1, 0, 1, 0),
                     c(1, 0, 1, 0, 0),
                     c(0, 1, 0, 0, 0))
# ここでネットワークオブジェクトを作っている。
mynet <- network(mynetmatrix, matrix.type="adjacency", directed=F)
# 矢印は示さない
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5))

# 隣接行列と線の集合の確認
as.matrix(mynet) # 隣接行列
as.matrix(mynet, matrix.type = "edgelist") # 線の集合









#############################
# igraphを用いる

# 先のsnaパッケージと同じことを行う。
# ただし、コマンドやオプションの指定の仕方が変わってくる。
# ここでの、plot.igraphは
# snaのgplotと用法が異なるので、注意する。
# igraphでは、素直にコマンドを用いると、
# テキストでは出てこなかった、2本の線の概念が出るので、やや発展した議論となっている。

# コマンドのコンフリクトがあるので、snaパッケージを取り除いておく。
detach(package:sna)

# igraphパッケージを読み込む。
library(igraph)

# ●方向のない線を考える。
# はじめに、線の集合を定義する。
myedgelist <- rbind(c(1, 2))
# 次に、ネットワークオブジェクトを作る。
# ここで、方向性がないことを指定する。
mynet <- graph.edgelist(myedgelist, directed=F)

# 矢印に関するオプションはグラフオブジェクトで指定しているので、
# あらたに、plot.igraphでは指定しないという考えかたのようである。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) #隣接行列がテキストで学んだ形となっている。ここでは0の代わりに.で用いられている。
as_edgelist(mynet) #線は一つで、点の順序関係ない。


# ●方向のある線を考える。
# 線の集合を定義している
myedgelist <- rbind(c(1, 2))
# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- graph.edgelist(myedgelist, directed=T)
# ネットワークグラフを描画する。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet)
as_edgelist(mynet) #線は一つで、順序関係あり。



# ●方向のないグラフでは、線を(1, 2)から(2, 1)へ方向を逆にして、定義しても結果は同じｒ。
# 線の集合を定義している
myedgelist <- rbind(c(2, 1)) #ここで先とは異なりc(2, 1)となっていることに注意する。
# ここでネットワークオブジェクトを作っている。
# 方向性がないことを指定する。
mynet <- graph.edgelist(myedgelist, directed=F)

# ネットワークグラフを描画する。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 対称行列に変換されている。隣接行列がテキストで学んだ形となっている。
as_edgelist(mynet) #線は一つで、順序関係ないので(1, 2)という線で表現しなおされている。


# ●方向のある線を考える。
# 線の集合を定義している
myedgelist <- rbind(c(1, 2))
# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- graph.edgelist(myedgelist, directed=T)
# ネットワークグラフを描画する。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet)
as_edgelist(mynet) #線は一つで、順序関係あり。




# ●方向のある線を考える。
# 線の集合を定義している。ここでは、線を(1, 2)から(2, 1)へ方向を逆にして、結果が変わることを確認する。
myedgelist <- rbind(c(2, 1))
# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- graph.edgelist(myedgelist, directed=T)

# ネットワークグラフを描画する。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet)
as_edgelist(mynet) #線は一つで、線として(2, 1)が示されている。
# 方向のないネットワークのように、(1, 2)に変換されることはなく、順序が関係していることがわかる。




# 二つの線で示しても良い。
# 方向は考えない
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 1))

# ここでネットワークオブジェクトを作っている。
# 方向性がないことを指定する。
mynet <- graph.edgelist(myedgelist, directed=F)

# ネットワークグラフを描画する。ここでは2本の線が示される。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # ここでの隣接行列も、線の数の2が示されている。
as_edgelist(mynet) #方向のないネットワークでは、igraphでは二つの線として表現されている。



# 二つの線で示しても良い。
# 方向を考える。
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 1))

# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- graph.edgelist(myedgelist, directed=T)

# ネットワークグラフを描画する。
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 対称行列の形となっている。
as_edgelist(mynet) #線は二つで示されている。





# 図1.11のネットワークの描画
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 1),
                    c(3, 1),
                    c(3, 2),
                    c(4, 1),
                    c(4, 2),
                    c(5, 1),
                    c(5, 2)
)

# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- graph.edgelist(myedgelist, directed=T)

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 隣接行列
as_edgelist(mynet) # 線のリスト


# 図1.13のネットワークの描画
# 線の集合を定義している
myedgelist <- rbind(c(1, 2),
                    c(2, 3),
                    c(3, 4),
                    c(4, 1),
                    c(2, 5))


# ここでネットワークオブジェクトを作っている。
# 方向性があることを指定する。
mynet <- graph.edgelist(myedgelist, directed=F)

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 隣接行列
as_edgelist(mynet) # 線のリスト





##################################
# Adjacency Matrix

# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(0, 0))

# ここでネットワークオブジェクトを作っている。
# 方向なし。
mynet <- graph.adjacency(mynetmatrix, mode="undirected")
# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) #隣接行列が対称行列に変換されている。
as_edgelist(mynet) #線は一つで、順序関係ない。




# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(0, 0))

# ここでネットワークオブジェクトを作っている。
# 方向あり。
mynet <- graph.adjacency(mynetmatrix, mode="directed")

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 隣接行列。
as_edgelist(mynet) # 線は隣接行列のとおり、1つとなっている。



# 離接行列を定義している
mynetmatrix <- rbind(c(0, 0),
                     c(1, 0))

# ここでネットワークオブジェクトを作っている。
# 方向なし。
mynet <- graph.adjacency(mynetmatrix, mode="undirected")
# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) #隣接行列が対称行列に変換されている。
as_edgelist(mynet) #線は一つで、順序関係ない。




# 離接行列を定義している
mynetmatrix <- rbind(c(0, 0),
                     c(1, 0))

# ここでネットワークオブジェクトを作っている。
# 方向あり。
mynet <- graph.adjacency(mynetmatrix, mode="directed")

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 隣接行列。
as_edgelist(mynet) # 線は隣接行列のとおり、1つとなっている。




# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(1, 0))

# ここでネットワークオブジェクトを作っている。
mynet <- graph.adjacency(mynetmatrix, mode="undirected")

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列からは、方向の無い線が一つと認識されることに注意する。
# これは、以下の隣接行列と線の集合からも確認できる。

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet)
as_edgelist(mynet) #線は一つに集約されている。





# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1),
                     c(1, 0))

# ここでネットワークオブジェクトを作っている。
# 方向あり
mynet <- graph.adjacency(mynetmatrix, mode="directed")

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet)
as_edgelist(mynet) #線は二つで示されている。





# 図1.11のグラフ
# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1, 0, 0, 0),
                     c(1, 0, 0, 0, 0),
                     c(1, 1, 0, 0, 0),
                     c(1, 1, 0, 0, 0),
                     c(1, 1, 0, 0, 0))

# 方向ありの、ネットワークオブジェクトを作っている。
mynet <- graph.adjacency(mynetmatrix, mode="directed")

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 隣接行列
as_edgelist(mynet) # 線の集合



# 図1.13のグラフ
# 離接行列を定義している
mynetmatrix <- rbind(c(0, 1, 0, 1, 0),
                     c(0, 0, 1, 0, 1),
                     c(0, 1, 0, 1, 0),
                     c(1, 0, 1, 0, 0),
                     c(0, 1, 0, 0, 0))

# 方向なしの、ネットワークオブジェクトを作っている。
mynet <- graph.adjacency(mynetmatrix, mode="directed")

# ネットワークを描画する
plot.igraph(mynet)

# 隣接行列と線の集合の確認
as_adjacency_matrix(mynet) # 隣接行列
as_edgelist(mynet) # 線の集合

