#推移性の議論

# 推移性を求めるためのライブラリを読み込んでおく。
library(sna)


# 図3.12のネットワークの隣接行列
mynet <- matrix(c(
  0,1,1,1,
  1,0,1,0,
  1,1,0,0,
  0,0,1,0),
  nrow = 4,
  byrow = TRUE)


# 1章では、ネットワークオブジェクトの作成したが、
# snaでは隣接行列から直接にグラフを描くこともできるので、
# 隣接行列から直接グラフを描く。


# グラフの確認（矢印を示す）
gplot(mynet, gmode = "digraph", label =c(1, 2, 3, 4))



# 推移性を求める（興味深いので、はじめにテキストで示した定義にしたがって）

# 距離が2のパスを求める。
d2 <- mynet %*% mynet
diag(d2) <- 0 # 自分から自分へのパスは取り除いておく。（後で和をとるときに、考えないので。）
# 以下でノードiからノードjへの距離2のパスの数が(i, j)要素の数値となっている。
d2
# 全ての要素を足し合わせると距離が2のパス数となる。
sum(d2)

# (i)から(j)への距離2へのパスについて、
# (i)から(j)への直接のパスがあると隣接行列の要素が1となる。
# そうすると、d2と隣接行列mynetの各要素の積を考えると、
# 距離2で始点と終点を直接結んでいるものがある(つまり、推移性が成立しているノード)。

d2 * mynet

# 以上の要素を全て足し合わせると、テキストで述べた
# 「距離が2のパスが始点と終点を直接結ぶリンクを存在するものの数」となる。

sum(d2 * mynet)

# 以上より推移性が以下で得られる。

sum(d2 * mynet)/sum(d2)


# テキストで示した推移性の導出 (snaを用いる)
# 隣接行列から直接求めることができる。
# ネットワークオブジェクトに対しても求めることもできるが、
# 隣接行列との対応を直接把握できる方が理解しやすいので、
# 隣接行列から推移性を求める。

gtrans(mynet)
gtrans(mynet, mode="digraph") #オプション明示


# # 一応、ネットワークオブジェクトから求める時には、以下となる。
# mynetg <- network(mynet, matrix.type="adjacency", directed=T)
# gplot(mynetg, gmode = "digraph", label =c(1, 2, 3, 4))
# gtrans(mynetg)
# gtrans(mynetg, mode="digraph") #オプション明示



# 以下では igraphを用いるので、コマンドの競合を避けるため、snaを取り除く
detach(package:sna)

##################
#igraphでの分析
library(igraph)

# igraphでは無向ネットワークで推移性を考えるので、
# 無向ネットワークをあらためて作る(図3.15のネットワーク)。
mynet.nd <- matrix(c(
  0,1,1,1,
  1,0,1,0,
  1,1,0,1,
  1,0,1,0),
  nrow = 4,
  byrow = TRUE)

# グラフオブジェクトを作成する。
mynetg.nd <- graph_from_adjacency_matrix(mynet.nd, mode="undirected")
# グラフを確認する
plot(mynetg.nd)


# 推移性を求め方は、先と同じである。

# 隣接行列から距離が2のパスを求める。
d2 <- mynet.nd %*% mynet.nd
diag(d2) <- 0 # 自分から自分へのパスは取り除いておく。（後で和をとるときに、考えないので。）
# 以下でノードiからノードjへの距離2のパスの数が(i, j)要素の数値となっている。
d2
# 全ての要素を足し合わせると距離が2のパス数となる。
sum(d2)

# (i)から(j)への距離2へのパスについて、
# (i)から(j)への直接のパスがあると隣接行列の要素が1となる。
# そうすると、d2と隣接行列mynetの各要素の積を考えると、
# 距離2で始点と終点を直接結んでいるものがある(つまり、推移性が成立しているノード)。

d2 * mynet.nd

# 以上の要素を全て足し合わせると、テキストで述べる
# 「距離が2のパスが始点と終点を直接結ぶリンクを存在するものの数」となる。

sum(d2 * mynet.nd)

# 以上より推移性が以下で得られる。

sum(d2 * mynet.nd)/sum(d2)



# igraphで求める。
transitivity(mynetg.nd, type="global")


# 有向グラフでも、無向グラフとして計算されることの確認
mynetg <- graph_from_adjacency_matrix(mynet, mode="directed")
plot(mynetg)
transitivity(mynetg, type="global") # undirectedで計算される。





###############################
# 以下でクラスター係数の議論をする。
# ここでは、無方向で考えている。

plot(mynetg.nd)


###########################
# ノード別のクラスター係数



# コマンドでクラスター係数を求める。
transitivity(mynetg.nd, type="local")


# ネットワーク全体のクラスター係数（ノード別のクラスター係数の平均値）
mean(transitivity(mynetg.nd, type="local"))



