#相互性の議論

# 相互性を求めるためのライブラリを読み込んでおく。
library(sna)


# 図3.12のネットワークの隣接行列
mynet <- matrix(c(
0,1,1,1,
1,0,1,0,
1,1,0,0,
0,0,1,0),
nrow = 4,
byrow = TRUE)

# 1章では、ネットワークオブジェクトの作成したが、
# snaでは隣接行列から直接にグラフを描くこともできるので、
# 隣接行列から直接グラフを描く。

# グラフの確認（矢印を示す）
gplot(mynet, gmode = "digraph", label =c(1, 2, 3, 4))


# テキストで示した相互性
# つまり、
# 分子：相互性を満たすリンクの数
# #{(1)<->(2), (1)<->(3), (2)<->(3)}
# 分母：存在するリンクの数
# #{(1)<->(2), (1)<->(3), (1)->(4), (2)<->(3), (4)->(3)}
# 以上から3/5 =0.6 で求めた。


# テキストで示した相互性の導出 (snaを用いる)
# 隣接行列から直接求めることができる。
# ネットワークオブジェクトに対しても求めることもできるが、
# 隣接行列との対応を直接把握できる方が理解しやすいので、
# 隣接行列から相互性を求める。
grecip(mynet, measure = "dyadic.nonnull")


# # 一応、ネットワークオブジェクトから求める時には、以下となる。
# mynetg <- network(mynet, matrix.type="adjacency", directed=T)
# gplot(mynetg, gmode = "digraph", label =c(1, 2, 3, 4))
# grecip(mynetg, measure = "dyadic.nonnull")


# テキストで示した相互性の導出 (igraphを用いる)

# コマンドの競合を避けるため、snaを取り除く
detach(package:sna)
library(igraph)

# グラフオブジェクトを作成する。
mynetg <- graph_from_adjacency_matrix(mynet, mode="directed")
# グラフを確認する
plot(mynetg, label =c(1, 2, 3, 4))

reciprocity(mynetg, mode =　"ratio")

# 他の相互性の導出 (1)
# 相互性の分析(snaでの対称なノードに注目)
# リンクがあるなしにかかわらず、対称な関係のノードに注目する。
# ここでは、(2)(4)[共にリンクが無いという意味で], (1)<->(2), (1)<->(3), (2)<->(3)
# という4つのノードの組である。
# 存在しうる、全てのノードの組は4*3/2 =6となっている。
# 以上より、ここでの相互性が4/6 =0.666..で示される。
# これはsnaでオプションをdyadicにすることで得られる。

detach(package:igraph)
library(sna)

grecip(mynet, measure = "dyadic")

# 他の相互性の導出 (2)
#相互性の分析(igraphでの個々のリンクに注目)
# ここでは(1)<->(2)を、(1)->(2)と(2)->(1)という二つのリンクとしてとらえる。
# すると、相互性を満たすリンクは
# (1)->(2), (2)->(1), (1)->(3), (3)->(1), (2)->(3), (3)->(2)
# となり、
# 存在するリンクは
# (1)->(4), (4)->(3), 
# (1)->(2), (2)->(1), (1)->(3), (3)->(1), (2)->(3), (3)->(2)
# 以上より、比率は、6/8 =0.75
# を得る。
# これはigraphでオプションをdefaultにすることで得られる。

detach(package:sna)
library(igraph)

reciprocity(mynetg, mode =　"default")

# あとで、snaを使う場合は以下で、igraphを取り除いておく。
detach(package:igraph)




