# 必要であれば、ggplot2をインストールしておく。
# install.packages("ggplot2")

#######################################################################
# c >d という条件で、円形ネットワークで、ペアワイズ安定となることの確認


# n= 5の円形ネットワークを考える。

# 円形ネットワークを考える。
# 各ノードは次の利得を得ている。
# 直接の利得が2つのノードから、
# 間接の利得が2つのノードから、距離が2なので次のようになる。
# リンクは2つのなのでコストは2cとなる。
# まとめると、
# 2*(d-c) +2*(d^2)
# となる。

# いまリンクを切ったとしよう。
# このときには、
# 直接の利得が1つのノードから、
# 間接の利得が3つのノードから得られ、距離はそれぞれ、2, 3, 4となる。
# リンクは1つのなのでコストはcとなる。
# まとめると、
# d +d^2 +d^3 +d^4 -c
# となる。


# 以上より、リンクを切るインセンティブがない条件（ペアワイズ安定の一つ目の条件）は、
# 2*d +2*(d^2) -2c >= d +d^2 +d^3 +d^4 -c
# <==> (d -c) +d^2 -d^3 -d^4 >= 0
# <==>  d +d^2 -d^3 -d^4 >= c

# ここでd^2 -d^3 -d^4に注目すると、dが小さい値だと
# d^2 -d^3 -d^4 が正の値をとる。
# このことは、dを0から0.7まで変化させてグラフを描くとわかる。

# 描きたい関数を定義する
myf <- function(d){d^2 -d^3 -d^4}

# 追加的なパッケージを使わず、curveを用いての描写
curve(myf, 0, 0.7)

# データの可視化において標準的なggplotを使った場合。
library(ggplot2)

# x軸の変数名をdとして、dのとる値0から0.7に指定する。
base <- ggplot(data.frame(d = c(0, 0.7)), aes(x=d))

# グラフを描画する。myfは先に定義した関数で、文字のサイズもやや大きくしている。
base + stat_function(fun=myf)  +theme_gray(base_size = 20)




# 以上より、dが0.4前後で、d^2 -d^3 -d^4は正となることがわかった。
# 例えば、d=0.4の時には、その値は0.0704となることが以下でわかる。
myf(0.4)

# したがって、d=0.4の時には、d +d^2 -d^3 -d^4は0.4704となる。
# これは以下で確認できる。
d <- 0.4
d +d^2 -d^3 -d^4

# これより、d=0.4, c=0.41としよう。
# このとき、d <c が成立している。
# さらに、
# d +d^2 -d^3 -d^4=0.4704であり、
# c=0.41であり、
# d +d^2 -d^3 -d^4 >= c が成立する。

# つまり、
# d <cの条件下でも、リンクを切るインセンティブはない。
# ペアワイズ安定の一つ目の条件が満たされる。

# つづいて、円形ネットワークを前提として、追加のリンクを張ってみよう。
# 追加のリンクを張れるノードは、二つあるが、どちらに張っても利得は以下で同じである。
# 直接の利得が3つのノードからでコストと合わせて、それぞれ(d -c)の利得を得ている。
# 間接の利得が1つノードからで、距離は2となる。
# したがって、リンクを追加しての利得は、
# 3*(d -c) +d^2
# である。

# 以上より、追加してリンクを張るインセンティブがない条件（ペアワイズ安定の二つ目の条件）は、
# 2*d +2*(d^2) -2c >= 3*(d -c) +d^2
# <==> -d +d^2 +c >= 0
# <==>  c >= d -d^2
# となる。

# いまは c >d を仮定しており、-d^2は必ず負なので、以上の条件は成立する。

# リンクを追加で張られた側の利得も同様の利得となり、つまり追加のリンクで利得は減少する。
# したがって、ペアワイズ安定の二つ目の「もし」の部分が成立しない。
# したがって、ペアワイズ安定の二つ目の条件と矛盾しないという意味で、満たされており、
# d <cの条件下でも、追加でリンクを張るインセンティブはない。
# 先に仮定した、d=0.4, c=0.41においても、d <cであり、追加でリンクを張るインセンティブはない。

# 以上より、主体数が5の円形ネットワークにおいて、
# d=0.4, c=0.41とするとき、c >d という条件となるにも関わらず、
# 円形ネットワークが、ペアワイズ安定となる。

