# detach("package:igraph", unload=TRUE)
library(sna)


######################
# 6.2.2 節の図について

##################
# 隣接行列の例（図6.4, 左）
mynet <- matrix(c(0, 1, 0, 1, 
              1, 0, 1, 0, 
              0, 1, 0, 1, 
              1, 0, 1, 0), nrow=4, ncol=4, byrow = T)

# SNAでは隣接行列をそのままグラフとして扱えるので、
# 以下では、ネットワークオブジェクトに変えることなく、
# 直接隣接行列からグラフや記述統計を求める。

# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4))

##############
# snaでの求め方
# 平均次数
sum(degree(mynet, gmode="graph"))/ncol(mynet)

# 密度
gden(mynet)

# 計算式からの導出
sum(degree(mynet, gmode="graph"))/(ncol(mynet)*(ncol(mynet) -1))


##############
# igraphでの求め方

detach("package:sna", unload=TRUE)
library("igraph")

# igraphでは，directed, undirectedはグラフオブジェクトを作るときに指定する。
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)


###############
# 平均次数
sum(degree(mynet_i))/vcount(mynet_i)


##############
# 密度
edge_density(mynet_i)

# 計算式から導出
(sum(degree(mynet_i)))/((vcount(mynet_i)*(vcount(mynet_i)-1))) # Non-directed



# snaを再読み込み
detach("package:igraph", unload=TRUE)
library("sna")



##################
# 隣接行列の例（図6.4, 右）
mynet <- matrix(c(
  # 1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16
  0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, # 1 
  1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, # 2 
  0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, # 3 
  1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, # 4 
  1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, # 5 
  0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, # 6 
  1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, # 7 
  0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, # 8 
  0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, # 9 
  0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, # 10 
  0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, # 11 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, # 12 
  0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, # 13 
  0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, # 14 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, # 15 
  0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0  # 16
), nrow=16, ncol=16, byrow = T)


# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1:16))

##############
# snaでの求め方
# 平均次数
sum(degree(mynet, gmode="graph"))/ncol(mynet)

# 密度
gden(mynet)

# 計算式からの導出
sum(degree(mynet, gmode="graph"))/(ncol(mynet)*(ncol(mynet) -1))


##############
# igraphでの求め方

detach("package:sna", unload=TRUE)
library("igraph")

# igraphでは，directed, undirectedはグラフオブジェクトを作るときに指定する。
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)


###############
# 平均次数
sum(degree(mynet_i))/vcount(mynet_i)


##############
# 密度
edge_density(mynet_i)

# 計算式から導出
(sum(degree(mynet_i)))/((vcount(mynet_i)*(vcount(mynet_i)-1))) # Non-directed





######################
# 6.3.2 節の図について

detach("package:igraph", unload=TRUE)
library("sna")


#########################################
# snaでの分析
#########################################

############
# 隣接行列の例（図6.7, 上段左端）

mynet <- matrix(c(0, 1, 1, 1, 1, 1, 
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0
              ), nrow=6, ncol=6, byrow = T)

# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5, 6))



# 分散を求める
# 次数
mydeg <- degree(mynet, gmode="graph") # 次数、つまり各列もしくは各行の総和
mydeg 

# 次数の平均
mean(mydeg)

# 次数の平均からの偏差
mydeg -mean(mydeg)

# 次数の平均からの偏差の2乗
(mydeg -mean(mydeg))^2

# 次数の平均からの偏差の2乗の平均としての分散
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/ncol(mynet)



# 中心化を計算式から導出する。
myn <- nrow(mynet)
totalmaxmyn <- (myn -1)*(myn -2)
totalmaxmyn

maxmyn <- max(degree(mynet, gmode="graph"))

sum(maxmyn -degree(mynet, gmode="graph"))/totalmaxmyn

# コマンドでの導出
centralization(mynet, degree, mode="graph")





###################
# igraphでグラフと分散と中心化を求める

detach("package:sna", unload=TRUE)
library("igraph")


# igraphのグラフオブジェクトの作成
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)

# 分散を求める
mydeg <- degree(mynet_i) # 次数、つまり各列もしくは各行の総和
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/vcount(mynet_i)


# 中心化
centr_degree(mynet_i, loops=FALSE)$centralization





############
# 隣接行列の例（図6.7, 上段中央）

detach("package:igraph", unload=TRUE)
library("sna")


mynet <- matrix(c(0, 1, 1, 1, 1, 0, 
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 1,
              0, 0, 0, 0, 1, 0
              ), nrow=6, ncol=6, byrow = T)

#########################################
# snaでの分析

# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5, 6))


# 分散を求める
# 次数の平均からの偏差の2乗の平均としての分散
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
mydeg <- degree(mynet, gmode="graph") # 次数、つまり各列もしくは各行の総和
sum((mydeg -mean(mydeg))^2)/ncol(mynet)


# 中心化を計算式から導出する。
myn <- nrow(mynet)
totalmaxmyn <- (myn -1)*(myn -2)
maxmyn <- max(degree(mynet, gmode="graph"))
sum(maxmyn -degree(mynet, gmode="graph"))/totalmaxmyn

# コマンドでの導出
centralization(mynet, degree, mode="graph")






###################
# igraphでグラフと分散と中心化を求める

detach("package:sna", unload=TRUE)
library("igraph")

mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)

# 分散を求める
mydeg <- degree(mynet_i) # 次数、つまり各列もしくは各行の総和
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/vcount(mynet_i)

# 中心化
centr_degree(mynet_i, loops=FALSE)$centralization



############
# 隣接行列の例（図6.7, 上段右端）

detach("package:igraph", unload=TRUE)
library("sna")


mynet <- matrix(c(0, 1, 1, 1, 0, 0, 
              1, 0, 0, 0, 1, 1,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              0, 1, 0, 0, 0, 0,
              0, 1, 0, 0, 0, 0
), nrow=6, ncol=6, byrow = T)

#########################################
# snaでの分析

# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5, 6))


# 分散を求める
# 次数の平均からの偏差の2乗の平均としての分散
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
mydeg <- degree(mynet, gmode="graph") # 次数、つまり各列もしくは各行の総和
sum((mydeg -mean(mydeg))^2)/ncol(mynet)


# 中心化を計算式から導出する。
myn <- nrow(mynet)
totalmaxmyn <- (myn -1)*(myn -2)
maxmyn <- max(degree(mynet, gmode="graph"))
sum(maxmyn -degree(mynet, gmode="graph"))/totalmaxmyn

# コマンドでの導出
centralization(mynet, degree, mode="graph")



detach("package:sna", unload=TRUE)
library("igraph")

################################
# igraphでグラフと分散と中心化を求める。

# igraphのグラフオブジェクトの作成
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)

# 分散を求める
mydeg <- degree(mynet_i) # 次数、つまり各列もしくは各行の総和
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/vcount(mynet_i)

# 中心化
centr_degree(mynet_i, loops=FALSE)$centralization




#########################################
# snaでの分析

############
# 隣接行列の例（図6.7, 下段左端）

detach("package:igraph", unload=TRUE)
library("sna")


mynet <- matrix(c(0, 1, 1, 1, 0, 0, 
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 0, 0,
              1, 0, 0, 0, 1, 0,
              0, 0, 0, 1, 0, 1,
              0, 0, 0, 0, 1, 0
), nrow=6, ncol=6, byrow = T)


# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5, 6))


# 分散を求める
# 次数の平均からの偏差の2乗の平均としての分散
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
mydeg <- degree(mynet, gmode="graph") # 次数、つまり各列もしくは各行の総和
sum((mydeg -mean(mydeg))^2)/ncol(mynet)


# 中心化を計算式から導出する。
myn <- nrow(mynet)
totalmaxmyn <- (myn -1)*(myn -2)
maxmyn <- max(degree(mynet, gmode="graph"))
sum(maxmyn -degree(mynet, gmode="graph"))/totalmaxmyn

# コマンドでの導出
centralization(mynet, degree, mode="graph")




################################
# igraphでグラフと分散と中心化を求める。

detach("package:sna", unload=TRUE)
library("igraph")

# igraphのグラフオブジェクトの作成
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)

# 分散を求める
mydeg <- degree(mynet_i) # 次数、つまり各列もしくは各行の総和
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/vcount(mynet_i)

# 中心化
centr_degree(mynet_i, loops=FALSE)$centralization







detach("package:igraph", unload=TRUE)
library("sna")

#########################################
# snaでの分析

############
# 隣接行列の例（図6.7, 下段中央）

mynet <- matrix(c(0, 1, 0, 0, 0, 0, 
              1, 0, 1, 0, 0, 0,
              0, 1, 0, 1, 0, 0,
              0, 0, 1, 0, 1, 0,
              0, 0, 0, 1, 0, 1,
              0, 0, 0, 0, 1, 0
), nrow=6, ncol=6, byrow = T)


# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5, 6))


# 分散を求める
# 次数の平均からの偏差の2乗の平均としての分散
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
mydeg <- degree(mynet, gmode="graph") # 次数、つまり各列もしくは各行の総和
sum((mydeg -mean(mydeg))^2)/ncol(mynet)


# 中心化を計算式から導出する。
myn <- nrow(mynet)
totalmaxmyn <- (myn -1)*(myn -2)
maxmyn <- max(degree(mynet, gmode="graph"))
sum(maxmyn -degree(mynet, gmode="graph"))/totalmaxmyn

# コマンドでの導出
centralization(mynet, degree, mode="graph")






################################
# igraphでグラフと分散と中心化を求める。

detach("package:sna", unload=TRUE)
library("igraph")

# igraphのグラフオブジェクトの作成
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)

# 分散を求める
mydeg <- degree(mynet_i) # 次数、つまり各列もしくは各行の総和
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/vcount(mynet_i)

# 中心化
centr_degree(mynet_i, loops=FALSE)$centralization




detach("package:igraph", unload=TRUE)
library("sna")


#########################################
# snaでの分析

############
# 隣接行列の例（図6.7, 下段右端）

mynet <- matrix(c(0, 1, 0, 0, 0, 1, 
              1, 0, 1, 0, 0, 0,
              0, 1, 0, 1, 0, 0,
              0, 0, 1, 0, 1, 0,
              0, 0, 0, 1, 0, 1,
              1, 0, 0, 0, 1, 0
), nrow=6, ncol=6, byrow = T)



# 矢印は示さないグラフ
gplot(mynet, gmode = "graph", label =c(1, 2, 3, 4, 5, 6))


# 分散を求める
# 次数の平均からの偏差の2乗の平均としての分散
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
mydeg <- degree(mynet, gmode="graph") # 次数、つまり各列もしくは各行の総和
sum((mydeg -mean(mydeg))^2)/ncol(mynet)


# 中心化を計算式から導出する。
myn <- nrow(mynet)
totalmaxmyn <- (myn -1)*(myn -2)
maxmyn <- max(degree(mynet, gmode="graph"))
sum(maxmyn -degree(mynet, gmode="graph"))/totalmaxmyn

# コマンドでの導出
centralization(mynet, degree, mode="graph")


################################
# igraphでグラフと中心化を求める。

detach("package:sna", unload=TRUE)
library("igraph")


# igraphのグラフオブジェクトの作成
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
plot(mynet_i)

# 分散を求める
mydeg <- degree(mynet_i) # 次数、つまり各列もしくは各行の総和
# Rのコマンドのvarは$n-1$で割る不偏分散なので、直接求めた。
sum((mydeg -mean(mydeg))^2)/vcount(mynet_i)

# 中心化
centr_degree(mynet_i, loops=FALSE)$centralization




############################################
# 以下は参考までに、
# 次数に関係するコマンドについて
# 隣接行列のどの部分と対応しているかをまとめておく。

library(igraph)

###################################################
# 隣接行列の例
mynet <- matrix(c(0, 1, 0, 0, 
              1, 0, 1, 1, 
              0, 1, 0, 1, 
              0, 1, 1, 0), nrow=4, ncol=4, byrow = T)



###########################################
# igraphでの分析
# igraphでは，directed, undirectedはグラフオブジェクトを作るときに指定する。
mynet_id <- graph_from_adjacency_matrix(mynet, mode = "directed")
mynet_i <- graph_from_adjacency_matrix(mynet, mode = "undirected")
par(mfrow = c(1, 2))
plot(mynet_id)
plot(mynet_i)


# 総リンク数 (directedの場合)
gsize(mynet_id)  # the number of edges, 隣接行列の1の要素の数
sum(degree(mynet_id)) # 隣接行列の1の要素の数の2倍 (in + out)
sum(degree(mynet_id, mode="all")) # 隣接行列の1の要素の数の2倍 (in + out)
sum(degree(mynet_id, mode="out")) # 隣接行列の1の要素の数
sum(degree(mynet_id, mode="in")) # 隣接行列の1の要素の数
sum(degree(mynet_id, mode="total")) # 隣接行列の1の要素の数の2倍 (in + out)


# 総リンク数 (undirectedの場合)
gsize(mynet_i)  # the number of edges, 上半分の隣接行列の1の要素の数
degree(mynet_i) # 各列もしくは各行の総和が各ノードの次数
degree(mynet_i, mode="all")# 各列もしくは各行の総和が各ノードの次数
sum(degree(mynet_i)) # (*上半分でない*) 隣接行列の1の要素の数


#########################################
# snaでの分析
#########################################
detach("package:igraph", unload=TRUE)
library("sna")

# 総リンク数について
mynet
degree(mynet, gmode="graph") # 各列もしくは各行の総和
sum(degree(mynet, gmode="graph")) # 隣接行列内の1の数
degree(mynet, gmode="digraph") # inとoutの総和
sum(degree(mynet, gmode="digraph")) # 隣接行列内の1の数の2倍(無向を両方向とみなす, in+out)
degree(mynet) # inとoutの総和
sum(degree(mynet)) # 隣接行列内の1の数の2倍 (inとoutの総和)







