############################################
# 図7.1（7.1.4項）による近接中心性の確認
# 図7.11（7.1.6項）による媒介中心性の確認

# 図7.1のネットワークの離接行列を定義している

                #1  2  3  4  5
mynet <- rbind(c(0, 1, 1, 1, 1), #1
               c(1, 0, 0, 0, 0), #2
               c(1, 0, 0, 0, 0), #3
               c(1, 0, 0, 0, 0), #4
               c(1, 0, 0, 0, 0)) #5

n <- ncol(mynet)

#####################
# igraphでの分析

#detach(package:sna) # snaを既に読み込んでいた場合は外す。競合があるので。
library(igraph)


# ここでネットワークオブジェクトを作っている。
mynet_i <- graph.adjacency(mynet, mode="undirected")

########################
# ネットワークを描画する
plot(mynet_i)

###############
# 次数中心性 (一応の確認)
degree(mynet_i)

###############
# 近接中心性

1/4 # 1/4は0.25
1/7 # 1/4は0.1428...

closeness(mynet_i) # 図内で示した標準化していない数値
closeness(mynet_i)*(n-1) #標準化したもの(一応の確認)

###############
# 媒介中心性
betweenness(mynet_i) # 図内で示した標準化していない数値
betweenness(mynet_i)/((n-1)*(n-2)/2) #標準化したもの(一応の確認)


#########################
# snaでの分析
detach(package:igraph) # snaを既に読み込んでいた場合は外す。競合があるので。
library(sna)

########################
# ネットワークを描画する

# 隣接行列をそのまま使える
gplot(mynet, gmode = "graph", label =c(1:n))

###############
# 次数中心性 (念のため)
degree(mynet, gmode="graph")

###############
# 近接中心性
# 以下が図内で示した標準化をしていないもの
closeness(mynet, gmode="graph", cmode="undirected")/(n -1) #標準化しないもの
# そのままのコマンドでは標準化がなされる。(一応の確認)
closeness(mynet, gmode="graph")


###############
# 媒介中心性
betweenness(mynet, gmode="graph", cmode="undirected")
betweenness(mynet, gmode="graph", cmode="undirected")/((n-1)*(n-2)/2) #標準化したもの(一応の確認)



#################################
# 図7.2, 7.4(次数中心性、近接中心性), 7.10(媒介中心性)で示された結果を確認する。

# 図7.2のネットワークの離接行列を定義している
                #1  2  3  4  5  6  7  8  9 10 11 12
mynet <- rbind(c(0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0), #1
               c(1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0), #2
               c(0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0), #3
               c(0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0), #4
               c(0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0), #5
               c(1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0), #6
               c(1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1), #7
               c(0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0), #8
               c(0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1), #9
               c(0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1), #10
               c(0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1), #11
               c(0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0)) #12

n <- ncol(mynet)

#####################
# igraphでの分析

# library(sna)
detach(package:sna) # snaを既に読み込んでいた場合は外す。競合があるので。
library(igraph)

# ここでネットワークオブジェクトを作っている。
mynet_i <- graph.adjacency(mynet, mode="undirected")

########################
# ネットワークを描画する
plot(mynet_i)

###############
# 次数中心性
degree(mynet_i)

###############
# 近接中心性
closeness(mynet_i) # 図内で示した標準化していない数値
closeness(mynet_i)*(n-1) #標準化したもの

###############
# 媒介中心性
betweenness(mynet_i) # 図内で示した標準化していない数値
betweenness(mynet_i)/((n-1)*(n-2)/2) #標準化したもの


###############
#　中心化

# 近接中心性
centr_clo(mynet_i)$centralization
# 媒介中心性
centr_betw(mynet_i)$centralization

# 近接中心性の中心化の直接の導出
myc <- closeness(mynet_i) # 図内で示した標準化していない数値
max(myc) # 最大値はこれで得られる
n <- length(myc) # ノード数をnに保存する

# 中心化の定義より、中心化の値を求める。
sum(max(myc)*rep(1, n) -myc)/((n -2)/(2*n -3))


# 媒介中心性の中心化の直接の導出
myc <- betweenness(mynet_i) # 図内で示した標準化していない数値
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -1)^2 *(n -2)/2)


#########################
# snaでの分析
detach(package:igraph) # snaを既に読み込んでいた場合は外す。競合があるので。
library(sna)

########################
# ネットワークを描画する

# 隣接行列をそのまま使える
gplot(mynet, gmode = "graph", label =c(1:n))

###############
# 次数中心性
degree(mynet, gmode="graph")

###############
# 近接中心性
# 以下が図内で示した標準化をしていないもの
closeness(mynet, gmode="graph", cmode="undirected")/(n -1) #標準化しないもの
# そのままのコマンドでは標準化がなされる。
closeness(mynet, gmode="graph")

###############
# 媒介中心性
betweenness(mynet, gmode="graph", cmode="undirected")
betweenness(mynet, gmode="graph", cmode="undirected")/((n-1)*(n-2)/2) #標準化したもの


###############
#　中心化

# 近接中心性
centralization(mynet, closeness, mode = "graph", cmode="undirected")
# 媒介中心性
centralization(mynet, betweenness, mode = "graph", cmode="undirected")


# 近接中心性の中心化の直接の導出
myc <- closeness(mynet, gmode="graph", cmode="undirected")/(n -1) #標準化しないもの
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -2)/(2*n -3))

# 媒介中心性の中心化の直接の導出
myc <- betweenness(mynet, gmode="graph", cmode="undirected")
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -1)^2 *(n -2)/2)



################################
# 図7.5のネットワークの離接行列を定義している

                #1  2  3  4  5  6
mynet <- rbind(c(0, 1, 1, 1, 0, 0), #1
               c(1, 0, 1, 0, 0, 0), #2
               c(1, 1, 0, 0, 0, 0), #3
               c(1, 0, 0, 0, 1, 1), #4
               c(0, 0, 0, 1, 0, 1), #5
               c(0, 0, 0, 1, 1, 0)) #6

n <- ncol(mynet)


#####################
# igraphでの分析

detach(package:sna) # snaを既に読み込んでいた場合は外す。競合があるので。
library(igraph)


# ここでネットワークオブジェクトを作っている。
mynet_i <- graph.adjacency(mynet, mode="undirected")

########################
# ネットワークを描画する
plot(mynet_i)

###############
# 次数中心性
degree(mynet_i)

###############
# 近接中心性
closeness(mynet_i) # 図内で示した標準化していない数値
closeness(mynet_i)*(n-1) #標準化したもの

###############
# 媒介中心性
betweenness(mynet_i) # 図内で示した標準化していない数値
betweenness(mynet_i)/((n-1)*(n-2)/2) #標準化したもの


###############
#　中心化

# 近接中心性
centr_clo(mynet_i)$centralization
# 媒介中心性
centr_betw(mynet_i)$centralization

# 近接中心性の中心化の直接の導出
myc <- closeness(mynet_i) # 図内で示した標準化していない数値
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -2)/(2*n -3))


# 媒介中心性の中心化の直接の導出
myc <- betweenness(mynet_i) # 図内で示した標準化していない数値
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -1)^2 *(n -2)/2)



#########################
# snaでの分析
detach(package:igraph) # snaを既に読み込んでいた場合は外す。競合があるので。
library(sna)

########################
# ネットワークを描画する

# 隣接行列をそのまま使える
gplot(mynet, gmode = "graph", label =c(1:n))

###############
# 次数中心性
degree(mynet, gmode="graph")

###############
# 近接中心性
# 以下が図内で示した標準化をしていないもの
closeness(mynet, gmode="graph", cmode="undirected")/(n -1) #標準化しないもの
# そのままのコマンドでは標準化がなされる。
closeness(mynet, gmode="graph")

###############
# 媒介中心性
betweenness(mynet, gmode="graph", cmode="undirected")
betweenness(mynet, gmode="graph", cmode="undirected")/((n-1)*(n-2)/2) #標準化したもの


###############
#　中心化

# 近接中心性
centralization(mynet, closeness, mode = "graph", cmode="undirected")
# 媒介中心性
centralization(mynet, betweenness, mode = "graph", cmode="undirected")


# 近接中心性の中心化の直接の導出
myc <- closeness(mynet, gmode="graph", cmode="undirected")/(n -1) #標準化しないもの
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -2)/(2*n -3))

# 媒介中心性の中心化の直接の導出
myc <- betweenness(mynet, gmode="graph", cmode="undirected")
max(myc)
n <- length(myc)

sum(max(myc)*rep(1, n) -myc)/((n -1)^2 *(n -2)/2)



