library("sna")

#########################################
# 固有ベクトル中心性を求める（対処法2の例）

# テキスト内のネットワークの隣接行列を得る。
G <- matrix(c(0, 1, 1, 1, 1, 
              0, 0, 1, 1, 0,  
              0, 0, 0, 0, 1,
              0, 1, 1, 0, 0, 
              1, 0, 0, 0, 0), nrow=5, ncol=5, byrow = T)

# ネットワークを描画する。
gplot(G, gmode="digraph", label=c(1: ncol(G)))

# 以下の固有値に関しては、snaパッケージを読まなくても求められる。
# 固有値関係の情報を得る。
myeigen <- eigen(G)
# 固有値をえる。
myeigen$values
# 固有値の絶対値（固有値は値の順に並んでいる）
abs(myeigen$values)
# 固有ベクトル（固有値の順に並ぶ）
myeigen$vectors
# 最大の固有値に対応するベクトル
myeigen$vectors[, 1]
# プラスにする。
-myeigen$vectors[, 1]

# 総和を1とするように値を調整する
-myeigen$vectors[, 1]/sum(-myeigen$vectors[, 1])


#####################################
# コマンドでの導出
# snaで求める。

?evcent #ヘルプの表示
evcent(G)

# 総和を1とするように値を調整する
evcent(G)/sum(evcent(G))
# コマンドで直接行うと
evcent(G, rescale = TRUE)

###################
# igraphでの導出
# igraphでは、転置をしないと、対応した固有ベクトル中心性が出ない

detach("package:sna", unload=TRUE)
library("igraph")

# ネットワークのオブジェクトを作成する
myg <- graph_from_adjacency_matrix(G)
# ネットワークを描画する。
plot(myg)

tG <- t(G) # 隣接行列Gを転置している
myg <- graph_from_adjacency_matrix(tG)

myeigen2 <- eigen_centrality(myg, directed = TRUE, scale = FALSE)$vector
myeigen2

# 総和を1とするように値を調整する
myeigen2/sum(myeigen2)

# eigen_centralityのscaleをTRUEとすると、最大の値を1とするように調整する

?eigen_centrality
eigen_centrality(myg, directed = TRUE, scale = TRUE)$vector


#########################################
# 対処法1での修正（固有値ベクトルも求める）
G <- matrix(c(0, 1/2, 1/3, 1/2, 1/2, 
              0,   0, 1/3, 1/2,   0,  
              0,   0,   0,   0, 1/2,
              0, 1/2, 1/3,   0,   0, 
              1,   0,   0,   0,   0), nrow=5, ncol=5, byrow = T)


# 固有値関係の情報を得る。
myeigen <- eigen(G)
# 固有値をえる。
myeigen$values
# 固有値の絶対値（固有値は値の順に並んでいる）
abs(myeigen$values)
# 固有ベクトル（固有値の順に並ぶ）
myeigen$vectors
# 最大の固有値に対応するベクトル
myeigen$vectors[, 1]

# 総和を1とするように値を調整する
myeigen$vectors[, 1]/sum(myeigen$vectors[, 1])



#####################################
# コマンドでの導出

# snaでのコマンドでの導出
detach("package:igraph", unload=TRUE)
library("sna")

# 固有ベクトル中心性
evcent(G)



###################
# igraphでの導出
# igraphでは、転置をしないと、対応した固有ベクトル中心性が出ない

detach("package:sna", unload=TRUE)
library("igraph")


tG <- t(G) # 隣接行列Gを転置している。

# 隣接行列の各要素の値が0もしくは1ではないので、weightedをTRUEにしている。
myg <- graph_from_adjacency_matrix(tG, weighted = TRUE)

myeigen2 <- eigen_centrality(myg, directed = TRUE, scale = FALSE)$vector
myeigen2






#################
# 収束しない状況

G <- matrix(c(0, 1, 0, 0, 
              0, 0, 1, 0, 
              0, 0, 0, 1, 
              1, 0, 0, 0), nrow=4, ncol=4, byrow = T)

# snaでネットワークを描く
detach("package:igraph", unload=TRUE)
library("sna")

# ネットワークを描画する。
gplot(G, gmode="digraph", label=c(1: ncol(G)))



# 固有値と固有ベクトルを求める。
# 固有ベクトルについては、テキストと符号や異なったり、
# 虚数(iや-i)を掛けて調整が必要な場合があるが、基本的な議論に変更はない。
myeigen <- eigen(G)
myeigen$values
abs(myeigen$values)
myeigen$vectors

c <- matrix(c(1,
              2, 
              3, 
              4), nrow=4, ncol=1, byrow = T)


# 初期ベクトルに隣接行列をどんどん掛けていく
c
G%*%c
G%*%G%*%c
G%*%G%*%G%*%c
G%*%G%*%G%*%G%*%c



#############################
# 無向のサークルネットワーク

G <- matrix(c(0, 1, 0, 1, 
              1, 0, 1, 0, 
              0, 1, 0, 1, 
              1, 0, 1, 0), nrow=4, ncol=4, byrow = T)

# ネットワークを描画する。
gplot(G, gmode="digraph", label=c(1: ncol(G)))

# 固有値と固有ベクトルを求める。
myeigen <- eigen(G)
myeigen$values
abs(myeigen$values)
myeigen$vectors

c <- matrix(c(1,
              2, 
              3, 
              4), nrow=4, ncol=1, byrow = T)



# 初期ベクトルに隣接行列をどんどん掛けていく
c
G%*%c
G%*%G%*%c
G%*%G%*%G%*%c
G%*%G%*%G%*%G%*%c

# 中心性の総和を1となるように値を調整する
(G%*%c)/sum(G%*%c)
(G%*%G%*%c)/sum(G%*%G%*%c)
(G%*%G%*%G%*%c)/sum(G%*%G%*%G%*%c)
(G%*%G%*%G%*%G%*%c)/sum(G%*%G%*%G%*%G%*%c)




myv<- G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%c
myv/sum(myv)
myv<- G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%c
myv/sum(myv)



################################################
# コンポーネントと固有ベクトル中心性について

#########################################
# 最大の固有値をもつコンポーネントのみが正の値となることについて


G <- matrix(c(0, 1, 0, 0, 1, 0, 0, 0,  
              0, 0, 1, 0, 0, 0, 0, 0,   
              0, 0, 0, 1, 0, 0, 0, 0, 
              1, 0, 0, 0, 0, 0, 0, 0, 
              0, 0, 0, 1, 0, 0, 0, 0, 
              0, 0, 0, 0, 0, 0, 1, 0, 
              0, 0, 0, 0, 0, 0, 0, 1, 
              0, 0, 0, 0, 0, 1, 0, 0 ), nrow=8, ncol=8, byrow = T)

G1 <- matrix(c(0, 1, 0, 0, 1,  
               0, 0, 1, 0, 0,   
               0, 0, 0, 1, 0,
               1, 0, 0, 0, 0, 
               0, 0, 0, 1, 0), nrow=5, ncol=5, byrow = T)

G2 <- matrix(c(0, 1, 0, 
               0, 0, 1, 
               1, 0, 0), nrow=3, ncol=3, byrow = T)


# snaでネットワークを描く
library("sna")

# ネットワークを描画する。
gplot(G, gmode="digraph", label=c(1: ncol(G)))
gplot(G1, gmode="digraph", label=c(1: ncol(G1)))
gplot(G2, gmode="digraph", label=c(1: ncol(G2)))

# 固有値と固有ベクトルを求める。
myeigen <- eigen(G)
myeigen1 <- eigen(G1)
myeigen2 <- eigen(G2)

# 固有値の絶対値について
abs(myeigen$values) #固有値は大きい順に並べられていることに注意する
abs(myeigen1$values) #固有値の最大は1.220744である
abs(myeigen2$values) #固有値の最大は1である


# 固有ベクトル中心性について
# 以下では、最後の3つのノードは0となっている。
# (正の要素とするために-1を掛けている。)
-myeigen$vectors[, 1] 

# Gのノード1から5までの固有ベクトル中心性と同じ値となる。
# (正の要素とするために-1を掛けている。)
-myeigen1$vectors[, 1] 

# 固有値は全て同じ絶対値で、
# 全ての固有ベクトルの要素を正にできるものは3番目の固有ベクトルなので。
-myeigen2$vectors[, 3]




# snaでのコマンドでの導出
# 固有ベクトル中心性
evcent(G) #ノード6から8までの数値が0.1の15乗という小さな数値で0ということが示されている。
evcent(G1)
evcent(G2)


#####################################
# igraphでのコマンドでの導出 (全体の隣接行列のみで)
# igraphでは、転置をしないと、対応した固有ベクトル中心性が出ない

detach("package:sna", unload=TRUE)
library("igraph")


tG <- t(G)
myg <- graph_from_adjacency_matrix(tG)
myeigen2 <- eigen_centrality(myg, directed = TRUE, scale = FALSE)$vector

myeigen2 #ノード6から8までの数値が0.0000000となっている。






#########################
# ノード数が少ないコンポーネントで正の固有ベクトル中心性となる例

G <- matrix(c(0, 1, 0, 0, 0, 0, 0,  
              0, 0, 1, 0, 0, 0, 0,   
              0, 0, 0, 1, 0, 0, 0, 
              1, 0, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0, 1, 1, 
              0, 0, 0, 0, 1, 0, 1, 
              0, 0, 0, 0, 1, 1, 0 ), nrow=7, ncol=7, byrow = T)

G1 <- matrix(c(0, 1, 0, 0,  
               0, 0, 1, 0,   
               0, 0, 0, 1, 
               1, 0, 0, 0), nrow=4, ncol=4, byrow = T)

G2 <- matrix(c(0, 1, 1, 
               1, 0, 1, 
               1, 1, 0), nrow=3, ncol=3, byrow = T)



# snaでネットワークを描く
detach("package:igraph", unload=TRUE)
library("sna")


# ネットワークを描画する。
gplot(G, gmode="digraph", label=c(1: ncol(G)))
gplot(G1, gmode="digraph", label=c(1: ncol(G1)))
gplot(G2, gmode="digraph", label=c(1: ncol(G2)))

# 固有値と固有ベクトルを求める。
myeigen <- eigen(G)
myeigen1 <- eigen(G1)
myeigen2 <- eigen(G2)

# 固有値の絶対値について
abs(myeigen$values) #固有値は大きい順に並べられていることに注意する
abs(myeigen1$values) #固有値の最大値は1である
abs(myeigen2$values) #固有値の最大値は2である


# 固有ベクトル中心性について
# 以下では、最初の4つのノードの値は0となっている。
myeigen$vectors[, 1]

# 全ての固有ベクトルの要素を正になるものは2番目の固有ベクトルなので。
myeigen1$vectors[, 2]

# Gのノード5から7までの固有ベクトル中心性と同じ値となる。
myeigen2$vectors[, 1]




#######################
# snaでのコマンドでの導出
# 固有ベクトル中心性
evcent(G) #ノード1から4までの数値が0.1の11乗という小さな数値で0ということが示されている。



#######################
# igraphでのコマンドでの導出 (全体の隣接行列のみで)
# igraphでは、転置をしないと、対応した固有ベクトル中心性が出ない

detach("package:sna", unload=TRUE)
library("igraph")


tG <- t(G)
myg <- graph_from_adjacency_matrix(tG)
myeigen2 <- eigen_centrality(myg, directed = TRUE, scale = FALSE)$vector

myeigen2 #ノード1から4までの数値が0.1の17乗という小さな数値で0ということが示されている。








################################################
# 収束しないという状況は、対称行列かつ列和を1としても解決しない
# 無向のサークルネットワークの列和を1にする。

G <- matrix(c(0,   1/2,   0, 1/2, 
              1/2,   0, 1/2,   0, 
              0,   1/2,   0, 1/2, 
              1/2,   0, 1/2,   0), nrow=4, ncol=4, byrow = T)

# この行列の一度出て戻ってくるのは、常に偶数回後なので、戻ってくる周期は2となる。

# 固有値と固有ベクトルを求める。
myeigen <- eigen(G)
myeigen$values
abs(myeigen$values)
myeigen$vectors


c <- matrix(c(1,
              2, 
              3, 
              4), nrow=4, ncol=1, byrow = T)


# 初期ベクトルに隣接行列をどんどん掛けていく
c
G%*%c
G%*%G%*%c
G%*%G%*%G%*%c
G%*%G%*%G%*%G%*%c


# 中心性の総和を1となるように値を調整する
myv<- c
myv/sum(myv)
myv<- G%*%c
myv/sum(myv)
myv<- G%*%G%*%c
myv/sum(myv)
myv<- G%*%G%*%G%*%c
myv/sum(myv)
myv<- G%*%G%*%G%*%G%*%c
myv/sum(myv)



myv<- G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%c
myv/sum(myv)
myv<- G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%G%*%c
myv/sum(myv)


#########################################
# ページランクを定義で求めることと、コマンドで求めることの比較
#########################################

# paerank以下はigraphで求めている。

library("igraph")


# ページランクでは実質的な流れの隣接行列を用いる。
# # 転置した隣接行列
Gp <-matrix(c(0, 0, 0, 0, 1,
             1, 0, 0, 1, 0,
             1, 1, 0, 1, 0,
             1, 1, 0, 0, 0,
             1, 0, 1, 0, 0), nrow=5, ncol=5, byrow = T)


myg <- graph_from_adjacency_matrix(Gp)
plot(myg)

# 中心性分割/移動の意味から行和を1とする。
hatGp <- Gp / rowSums(Gp)

# 右から中心性を掛けるので意味が合うように転置して、修正した推移確率行列を作る。
tildeGp <- t(hatGp) 
tildeGp

# ノイズの行列を作成する
n <- nrow(tildeGp) #ノード数
u <- matrix(1/n, nrow = n, ncol = n)
u

# ノイズ用の行列に対するパラメータを設定する。
e <- 0.85 

# ページランクで最終的に用いる修正された隣接行列を作成する。
checkG <- e * tildeGp + (1 - e) * u
checkG

# 固有ベクトルを求める
myeigen <- eigen(checkG)

# 最大固有値に対応する固有ベクトルをとり、要素も正の値にしておく。
mypage <- abs(myeigen$vectors[, 1])
mypage

# 中心性の総和を1となるように値を調整する。
mypage/sum(mypage)


# ページランクをコマンドで求める。
mypage.rank <- page_rank(myg, damping = 0.85)
mypage.rank$vector



