##############################
# ナッシュ均衡を求める。

G <- matrix(
  c(0, 1, 0, 0, 1, 0,
    1, 0, 1, 0, 1, 0,
    0, 1, 0, 0, 1, 1,
    0, 0, 0, 0, 1, 0,
    1, 1, 1, 1, 0, 0,
    0, 0, 1, 0, 0, 0), nrow=6, ncol=6, byrow =T)


I <- matrix(
  c(1, 0, 0, 0, 0, 0,
    0, 1, 0, 0, 0, 0,
    0, 0, 1, 0, 0, 0,
    0, 0, 0, 1, 0, 0,
    0, 0, 0, 0, 1, 0,
    0, 0, 0, 0, 0, 1), nrow=6, ncol=6, byrow =T)

one <-matrix(
  c(1,
    1,
    1,
    1,
    1,
    1)
)

b <- 0.3


# 以下計算過程の再現
I -b*G
b*one
solve(I -b*G)
solve(I -b*G) %*% (b*one)

# ナッシュ均衡
mynasheq <- solve(I -b*G) %*% (b*one)
mynasheq

# 総和を1となるように調整をする
mynasheq_adj <- mynasheq/(sum(mynasheq))
mynasheq_adj



######################
# ボナチッチ・パワー中心性を定義式から計算する
b      # betaにはbをそのまま用いる。
a <- 1 # alphaは1とする。

mybona <- solve(I -b*G) %*% (a*G %*% one)
mybona

mybona_adj <- mybona/(sum(mybona)) # 総和を1とするように調整した値。
mybona_adj

mybona_adj2 <- sqrt(6/sum(mybona^2))*mybona # 中心性の各値の二乗和がノード数となるように調整
mybona_adj2

      
######################
# ボナチッチ・パワー中心性からナッシュ均衡を再現する
((b^2)/1) * mybona +b*one
mynasheq


######################
# 注意：総和を1に調整した中心性では以上の等式は使えない。
((b^2)/1) * mybona_adj +b*one
mynasheq_adj



######################################
# snaでボナチッチ・パワー中心性を求める。
library(sna)

# ネットワークを描画する。
gplot(G, gmode="graph", label=c(1: ncol(G)))


# ボナチッチ・パワー中心性をコマンドで求める
# デフォルトでは、alphaは中心性の各値の二乗和がノード数となるように調整されている。
bonpow(G, exponent = b)

# 定義式から求めた値と比較がしやすいように、総和を1とするように調整するには以下とする。
bonpow(G, exponent = b, rescale=T)




##########################################
# igraphでボナチッチ・パワー中心性を確認する。

detach("package:sna", unload = TRUE)
library(igraph)


# ネットワークオブジェクトを作成する。
myg <- graph_from_adjacency_matrix(G, mode="undirected")

# ネットワークを描画する
plot(myg)

# ボナチッチ・パワー中心性をコマンドで求める
?power_centrality
# デフォルトでは、alphaは中心性の各値の二乗和がノード数となるように調整されている。
power_centrality(myg, exponent = b)

# 定義式から求めた値と比較がしやすいように、総和を1とするように調整するには以下とする。
power_centrality(myg, exponent = b, rescale=T)






